syms r00 r01 r02 r10 r11 r12 r20 r21 r22 t0 t1 t2 real
R = [r00 r01 r02; r10 r11 r12; r20 r21 r22]
t = [t0 t1 t2]'
% we have a rotation matrix and a translation vector

qw = sqrt(1 + r00 + r11 + r22) / 2;
qx = (r21 - r12) / (4 * qw);
qy = (r02 - r20) / (4 * qw);
qz = (r10 - r01) / (4 * qw);
% simplified matrix to quaternion (would need several branches
% for numerical stability)

%qnorm = (sqrt([qx qy qz] * [qx qy qz]')); % MATLAB trolls you if U no careful (unwanted conj() inserted despite real outputs)
qnorm = (sqrt(qx^2 + qy^2 + qz^2));
halfangle = asin(qnorm);
% get half of the rotation angle

ax = (qx / qnorm * 2 * halfangle);
ay = (qy / qnorm * 2 * halfangle);
az = (qz / qnorm * 2 * halfangle);
% get the rotation as axis-angle

vec = [ax ay az t0 t1 t2]
Rtvec = [r00 r01 r02 r10 r11 r12 r20 r21 r22 t0 t1 t2];
J = jacobian(vec, Rtvec);
% let matlab calculate the derivatives

quat_t = [ qx qy qz t0 t1 t2]
J0 = jacobian(quat_t, Rtvec);
%
syms ax_ ay_ az_ qx_ qy_ qz_ qw_ real
%qnorm_ = (sqrt([qx_ qy_ qz_] * [qx_ qy_ qz_]')); % MATLAB trolls you if U no careful (unwanted conj() inserted despite real outputs)
qnorm_ = (sqrt(qx_^2 + qy_^2 + qz_^2));
halfangle_ = asin(qnorm_);
ax_ = (qx_ / qnorm_ * 2 * halfangle_);
ay_ = (qy_ / qnorm_ * 2 * halfangle_);
az_ = (qz_ / qnorm_ * 2 * halfangle_);
vec_ = [ax_ ay_ az_ t0 t1 t2]
quat_t_ = [ qx_ qy_ qz_ t0 t1 t2]
J1 = jacobian(vec_, quat_t_);
J1_ = J1;
% calculate the jacobians separately (note how J1 is on separate variables)

% chain rule as:
%  f(   g(R))' =  f'(   g(R)) *    g'(R)
% aa(quat(R))' = aa'(quat(R)) * quat'(R)
%            J =           J1 *       J0

len_J0 = strcat(num2str(length(ccode(simple(J0))) / 1024), ' kB')
len_J1 = strcat(num2str(length(ccode(simple(J1))) / 1024), ' kB')

J1 = subs(J1, qx_, qx);
J1 = subs(J1, qy_, qy);
J1 = subs(J1, qz_, qz);
J1 = subs(J1, qw_, qw);
% substitute the quaternion to J1 (in practice, we would
% substitute *values* rather than *formulas*)

error = simplify(J - J1 * J0)
% verify
